@@ -1,7 +1,7 @@
 /*
  * This file is part of bino, a 3D video player.
  *
- * Copyright (C) 2006, 2007, 2009, 2010
+ * Copyright (C) 2006-2007, 2009-2011
  * Martin Lambers <marlam@marlam.de>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -356,7 +356,7 @@ namespace opt
                 {
                     for (size_t i = 0; i < _allowed_values.size(); i++)
                     {
-                        if (_allowed_values[i] == v)
+                        if (_allowed_values[i] <= v && _allowed_values[i] >= v)
                         {
                             ok = true;
                             break;
@@ -512,7 +512,7 @@ namespace opt
                 {
                     for (size_t j = 0; j < vv.size(); j++)
                     {
-                        if (_allowed_values[i] == vv[j])
+                        if (_allowed_values[i] <= vv[j] && _allowed_values[i] >= vv[j])
                         {
                             ok = true;
                             break;
