@@ -1,9 +1,10 @@
 /*
  * This file is part of bino, a 3D video player.
  *
- * Copyright (C) 2010
+ * Copyright (C) 2010-2011
  * Martin Lambers <marlam@marlam.de>
  * Stefan Eilemann <eile@eyescale.ch>
+ * Frédéric Devernay <Frederic.Devernay@inrialpes.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -71,6 +72,8 @@ int main(int argc, char *argv[])
     input_modes.push_back("even-odd-rows");
     opt::val<std::string> input_mode("input", 'i', opt::optional, input_modes, "");
     options.push_back(&input_mode);
+    opt::val<int> audio("audio", 'a', opt::optional, 1, 999, 1);
+    options.push_back(&audio);
     std::vector<std::string> video_output_modes;
     video_output_modes.push_back("mono-left");
     video_output_modes.push_back("mono-right");
@@ -99,6 +102,12 @@ int main(int argc, char *argv[])
     options.push_back(&swap_eyes);
     opt::flag benchmark("benchmark", 'b', opt::optional);
     options.push_back(&benchmark);
+    opt::val<float> parallax("parallax", 'P', opt::optional, -1.0f, +1.0f, 0.0f);
+    options.push_back(&parallax);
+    opt::tuple<float> crosstalk("crosstalk", 'C', opt::optional, 0.0f, 100.0f, std::vector<float>(3, 0.0f));
+    options.push_back(&crosstalk);
+    opt::val<float> ghostbust("ghostbust", 'G', opt::optional, 0.0f, 100.0f);
+    options.push_back(&ghostbust);
     // Accept some Equalizer options. These are passed to Equalizer for interpretation.
     opt::val<std::string> eq_server("eq-server", '\0', opt::optional);
     options.push_back(&eq_server);
@@ -112,6 +121,14 @@ int main(int argc, char *argv[])
     options.push_back(&eq_render_client);
 
     std::vector<std::string> arguments;
+#ifdef __APPLE__
+    // strip -psn* option when launching from the Finder on mac
+    if (argc > 1 && strncmp("-psn", argv[1], 4) == 0)
+    {
+        argc--;
+        argv++;
+    }
+#endif
     if (!opt::parse(argc, argv, options, 0, 3, arguments))
     {
         return 1;
@@ -119,7 +136,7 @@ int main(int argc, char *argv[])
     if (version.value())
     {
         msg::req("%s version %s", PACKAGE_NAME, VERSION);
-        msg::req("    Copyright (C) 2010 the Bino developers.");
+        msg::req("    Copyright (C) 2011 the Bino developers.");
         msg::req("    This is free software. You may redistribute copies of it");
         msg::req("    under the terms of the GNU General Public License.");
         msg::req("    There is NO WARRANTY, to the extent permitted by law.");
@@ -143,6 +160,7 @@ int main(int argc, char *argv[])
                 "  --version            Print version\n"
                 "  -g|--gui             Show the GUI (default if no files are given)\n"
                 "  -l|--log-level=LEVEL Set log level (debug, info, warning, error, quiet)\n"
+                "  -a|--audio=STREAM    Select audio stream (1-n, depending on the input)\n"
                 "  -i|--input=TYPE      Select input type (default autodetect):\n"
                 "    mono                 Single view\n"
                 "    separate             Left/right view in separate streams\n"
@@ -173,9 +191,13 @@ int main(int argc, char *argv[])
                 "  -f|--fullscreen      Fullscreen\n"
                 "  -c|--center          Center window on screen\n"
                 "  -s|--swap-eyes       Swap left/right view\n"
+                "  -P|--parallax=VAL    Parallax adjustment (-1 to +1)\n"
+                "  -C|--crosstalk=VAL   Crosstalk leak level in %% (0 to 100). VAL may\n"
+                "                       be one value or comma-separated R,G,B values.\n"
+                "  -G|--ghostbust=VAL   Amount of ghostbusting to apply, in %% (0 to 100).\n"
                 "  -b|--benchmark       Benchmark mode (no audio, no timesync, show fps)\n"
                 "\n"
-                "Keyboard control:\n"
+                "Interactive control:\n"
                 "  q or ESC             Quit\n"
                 "  p or SPACE           Pause / unpause\n"
                 "  f                    Toggle fullscreen\n"
@@ -185,15 +207,23 @@ int main(int argc, char *argv[])
                 "  3, 4                 Adjust brightness\n"
                 "  5, 6                 Adjust hue\n"
                 "  7, 8                 Adjust saturation\n"
+                "  <, >                 Adjust parallax\n"
+                "  (, )                 Adjust ghostbusting\n"
                 "  left, right          Seek 10 seconds backward / forward\n"
                 "  up, down             Seek 1 minute backward / forward\n"
-                "  page up, page down   Seek 10 minutes backward / forward",
+                "  page up, page down   Seek 10 minutes backward / forward\n"
+                "  Mouse click          Seek according to the horizontal click position",
                 program_name);
     }
     if (version.value() || help.value())
     {
         return 0;
     }
+    if (crosstalk.value().size() != 1 && crosstalk.value().size() != 3)
+    {
+        msg::err("invalid crosstalk levels");
+        return 1;
+    }
 
 #if HAVE_LIBEQUALIZER
     if (arguments.size() > 0 && arguments[0] == "--eq-client")
@@ -241,6 +271,7 @@ int main(int argc, char *argv[])
         init_data.log_level = msg::REQ;
     }
     init_data.filenames = arguments;
+    init_data.audio_stream = audio.value() - 1;
     if (input_mode.value() == "")
     {
         init_data.input_mode = input::automatic;
@@ -284,6 +315,11 @@ int main(int argc, char *argv[])
             debug::crash();
         }
     }
+    init_data.video_state.parallax = parallax.value();
+    init_data.video_state.crosstalk_r = crosstalk.value()[0] / 100.0f;
+    init_data.video_state.crosstalk_g = crosstalk.value()[crosstalk.value().size() == 3 ? 1 : 0] / 100.0f;
+    init_data.video_state.crosstalk_b = crosstalk.value()[crosstalk.value().size() == 3 ? 2 : 0] / 100.0f;
+    init_data.video_state.ghostbust = ghostbust.value() / 100.0f;
     init_data.video_state.fullscreen = fullscreen.value();
     init_data.video_state.swap_eyes = swap_eyes.value();
     if (center.value())
