@@ -1,9 +1,9 @@
 /*
  * This file is part of bino, a 3D video player.
  *
- * Copyright (C) 2010
+ * Copyright (C) 2010-2011
  * Martin Lambers <marlam@marlam.de>
- * Lion-Simba <lion-simba@pridelands.ru>
+ * Alexey Osipov <lion-simba@pridelands.ru>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,7 +37,10 @@
 
 
 player_init_data::player_init_data()
-    : log_level(msg::INF), benchmark(false), filenames(),
+    : log_level(msg::INF),
+    benchmark(false),
+    filenames(),
+    audio_stream(0),
     input_mode(input::automatic),
     video_mode(video_output::stereo),
     video_state(),
@@ -74,6 +77,14 @@ player::~player()
     }
 }
 
+float player::normalize_pos(int64_t pos)
+{
+    double pos_min = _start_pos + _input->initial_skip();
+    double pos_max = pos_min + _input->duration();
+    double npos = (pos_max > pos_min ? (static_cast<double>(pos) - pos_min) / (pos_max - pos_min) : 0.0f);
+    return npos;
+}
+
 void player::reset_playstate()
 {
     _running = false;
@@ -85,6 +96,7 @@ void player::reset_playstate()
     _quit_request = false;
     _pause_request = false;
     _seek_request = 0;
+    _set_pos_request = -1.0f;
 }
 
 void player::create_decoders(const std::vector<std::string> &filenames)
@@ -108,12 +120,13 @@ void player::create_decoders(const std::vector<std::string> &filenames)
     }
 }
 
-void player::create_input(enum input::mode input_mode)
+void player::create_input(enum input::mode input_mode, int selected_audio_stream)
 {
     int video_decoder[2] = { -1, -1 };
     int video_stream[2] = { -1, -1 };
     int audio_decoder = -1;
     int audio_stream = -1;
+    int skip_audio_streams = selected_audio_stream;
     for (size_t i = 0; i < _decoders.size(); i++)
     {
         for (int j = 0; j < _decoders[i]->video_streams(); j++)
@@ -139,6 +152,11 @@ void player::create_input(enum input::mode input_mode)
             {
                 continue;
             }
+            else if (skip_audio_streams > 0)
+            {
+                skip_audio_streams--;
+                continue;
+            }
             else
             {
                 audio_decoder = i;
@@ -146,6 +164,10 @@ void player::create_input(enum input::mode input_mode)
             }
         }
     }
+    if (selected_audio_stream > 0 && audio_stream == -1)
+    {
+        throw exc(str::asprintf("audio stream %d not found", selected_audio_stream + 1));
+    }
     _input = new input();
     _input->open(_decoders,
             video_decoder[0], video_stream[0],
@@ -154,14 +176,6 @@ void player::create_input(enum input::mode input_mode)
             input_mode);
 }
 
-void player::get_input_info(int *w, int *h, float *ar, enum decoder::video_frame_format *fmt)
-{
-    *w = _input->video_width();
-    *h = _input->video_height();
-    *ar = _input->video_aspect_ratio();
-    *fmt = _input->video_preferred_frame_format();
-}
-
 void player::create_audio_output()
 {
     if (_input->has_audio() && !_benchmark)
@@ -194,7 +208,7 @@ void player::open_video_output(enum video_output::mode video_mode, unsigned int
         }
     }
     _video_output->open(
-            _input->video_preferred_frame_format(),
+            _input->video_format(), _input->video_is_mono(),
             _input->video_width(), _input->video_height(), _input->video_aspect_ratio(),
             video_mode, _video_state, video_flags, -1, -1);
     _video_output->process_events();
@@ -225,98 +239,122 @@ void player::run_step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool
     else if (!_running)
     {
         // Read initial data and start output
-        if (_input->read_video_frame() < 0)
+        _video_pos = _input->read_video_frame();
+        if (_video_pos == std::numeric_limits<int64_t>::min())
         {
             msg::dbg("empty video input");
             notify(notification::play, true, false);
             return;
         }
-        _current_pos = 0;
-        _next_frame_pos = 0;
-        _sync_point_pos = 0;
-        _sync_point_av_offset = 0;
         if (_audio_output)
         {
             _audio_output->status(&_required_audio_data_size);
-            if (_input->read_audio_data(&_audio_data, _required_audio_data_size) < 0)
+            _audio_pos = _input->read_audio_data(&_audio_data, _required_audio_data_size);
+            if (_audio_pos == std::numeric_limits<int64_t>::min())
             {
                 msg::dbg("empty audio input");
                 notify(notification::play, true, false);
                 return;
             }
             _audio_output->data(_audio_data, _required_audio_data_size);
-            _sync_point_time = _audio_output->start();
+            _master_time_start = _audio_output->start();
+            _master_time_pos = _audio_pos;
+            _current_pos = _audio_pos;
         }
         else
         {
-            _sync_point_time = timer::get_microseconds(timer::monotonic);
+            _master_time_start = timer::get_microseconds(timer::monotonic);
+            _master_time_pos = _video_pos;
+            _current_pos = _video_pos;
         }
-        _fps_mark_time = _sync_point_time;
+        _start_pos = _current_pos;
+        _fps_mark_time = timer::get_microseconds(timer::monotonic);
         _frames_shown = 0;
         _running = true;
-        _need_frame = false;
-        _first_frame = true;
-        *more_steps = true;
-        *prep_frame = true;
-        return;
-    }
-    else if (_seek_request != 0)
-    {
-        if (_seek_request < 0 && -_seek_request > _current_pos)
+        if (_input->initial_skip() > 0)
         {
-            _seek_request = -_current_pos;
+            _seek_request = _input->initial_skip();
         }
-        if (_input->duration() > 0)
+        else
         {
-            if (_seek_request > 0 && _current_pos + _seek_request >= std::max(_input->duration() - 5000000, static_cast<int64_t>(0)))
+            _need_frame = false;
+            _first_frame = true;
+            *more_steps = true;
+            *prep_frame = true;
+            return;
+        }
+    }
+    if (_seek_request != 0 || _set_pos_request >= 0.0f)
+    {
+        int64_t old_pos = _current_pos;
+        if (_set_pos_request >= 0.0f)
+        {
+            int64_t dest_pos_min = _start_pos + _input->initial_skip();
+            int64_t dest_pos_max = dest_pos_min + _input->duration() - 2000000;
+            if (dest_pos_max <= dest_pos_min)
             {
-                _seek_request = std::max(_input->duration() - 5000000 - _current_pos, static_cast<int64_t>(0));
+                *seek_to = _current_pos;
+            }
+            else
+            {
+                *seek_to = static_cast<double>(_set_pos_request) * dest_pos_max
+                    + (1.0 - static_cast<double>(_set_pos_request)) * dest_pos_min;
             }
         }
-        *seek_to = _current_pos + _seek_request;
+        else
+        {
+            if (_current_pos + _seek_request < _start_pos + _input->initial_skip())
+            {
+                _seek_request = _start_pos + _input->initial_skip() - _current_pos;
+            }
+            if (_input->duration() > 0)
+            {
+                if (_seek_request > 0 && _current_pos + _seek_request >= std::max(_start_pos + _input->duration() - 2000000, static_cast<int64_t>(0)))
+                {
+                    _seek_request = std::max(_start_pos + _input->duration() - 2000000 - _current_pos, static_cast<int64_t>(0));
+                }
+            }
+            *seek_to = _current_pos + _seek_request;
+        }
         _seek_request = 0;
+        _set_pos_request = -1.0f;
         _input->seek(*seek_to);
-        _next_frame_pos = _input->read_video_frame();
-        if (_next_frame_pos < 0)
+
+        _video_pos = _input->read_video_frame();
+        if (_video_pos == std::numeric_limits<int64_t>::min())
         {
             msg::wrn("seeked to end of video?!");
             notify(notification::play, true, false);
             return;
         }
+        if (_audio_output)
+        {
+            _audio_output->stop();
+            _audio_output->status(&_required_audio_data_size);
+            _audio_pos = _input->read_audio_data(&_audio_data, _required_audio_data_size);
+            if (_audio_pos == std::numeric_limits<int64_t>::min())
+            {
+                msg::wrn("seeked to end of audio?!");
+                notify(notification::play, true, false);
+                return;
+            }
+            _audio_output->data(_audio_data, _required_audio_data_size);
+            _master_time_start = _audio_output->start();
+            _master_time_pos = _audio_pos;
+            _current_pos = _audio_pos;
+        }
         else
         {
-            if (_audio_output)
-            {
-                _audio_output->stop();
-                _audio_output->status(&_required_audio_data_size);
-                int64_t audio_pos = _input->read_audio_data(&_audio_data, _required_audio_data_size);
-                if (audio_pos < 0)
-                {
-                    msg::wrn("seeked to end of audio?!");
-                    notify(notification::play, true, false);
-                    return;
-                }
-                _audio_output->data(_audio_data, _required_audio_data_size);
-                _sync_point_time = _audio_output->start();
-                _sync_point_pos = audio_pos;
-                _sync_point_av_offset = _next_frame_pos - audio_pos;
-            }
-            else
-            {
-                _sync_point_pos = _next_frame_pos;
-                _sync_point_time = timer::get_microseconds(timer::monotonic);
-                _sync_point_av_offset = 0;
-            }
-            _fps_mark_time = _sync_point_time;
-            _frames_shown = 0;
-            notify(notification::pos, _current_pos / 1e6f, _sync_point_pos / 1e6f);
-            _need_frame = false;
-            _current_pos = _sync_point_pos;
-            _previous_frame_dropped = false;
-            *more_steps = true;
-            *prep_frame = true;
-            return;
+            _master_time_start = timer::get_microseconds(timer::monotonic);
+            _master_time_pos = _video_pos;
+            _current_pos = _video_pos;
         }
+        notify(notification::pos, normalize_pos(old_pos), normalize_pos(_current_pos));
+        _need_frame = false;
+        _previous_frame_dropped = false;
+        *more_steps = true;
+        *prep_frame = true;
+        return;
     }
     else if (_pause_request)
     {
@@ -338,8 +376,8 @@ void player::run_step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool
     }
     else if (_need_frame)
     {
-        _next_frame_pos = _input->read_video_frame();
-        if (_next_frame_pos < 0)
+        _video_pos = _input->read_video_frame();
+        if (_video_pos == std::numeric_limits<int64_t>::min())
         {
             if (_first_frame)
             {
@@ -357,6 +395,13 @@ void player::run_step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool
         {
             _first_frame = false;
         }
+        if (!_audio_output)
+        {
+            _master_time_start += (_video_pos - _master_time_pos);
+            _master_time_pos = _video_pos;
+            _current_pos = _video_pos;
+            notify(notification::pos, normalize_pos(_current_pos), normalize_pos(_current_pos));
+        }
         _need_frame = false;
         if (_drop_next_frame)
         {
@@ -379,9 +424,7 @@ void player::run_step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool
             }
             else
             {
-                _sync_point_time += timer::get_microseconds(timer::monotonic) - _pause_start;
-                _fps_mark_time = _sync_point_time;
-                _frames_shown = 0;
+                _master_time_start += timer::get_microseconds(timer::monotonic) - _pause_start;
             }
             _in_pause = false;
             notify(notification::pause, true, false);
@@ -390,33 +433,39 @@ void player::run_step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool
         if (_audio_output)
         {
             // Check if audio needs more data, and get audio time
-            _current_time = _audio_output->status(&_required_audio_data_size);
+            _master_time_current = _audio_output->status(&_required_audio_data_size) - _master_time_start
+                + _master_time_pos;
             // Output requested audio data
             if (_required_audio_data_size > 0)
             {
-                if (_input->read_audio_data(&_audio_data, _required_audio_data_size) < 0)
+                _audio_pos = _input->read_audio_data(&_audio_data, _required_audio_data_size);
+                if (_audio_pos == std::numeric_limits<int64_t>::min())
                 {
                     msg::dbg("end of audio stream");
                     notify(notification::play, true, false);
                     return;
                 }
+                _master_time_start += (_audio_pos - _master_time_pos);
+                _master_time_pos = _audio_pos;
                 _audio_output->data(_audio_data, _required_audio_data_size);
+                _current_pos = _audio_pos;
+                notify(notification::pos, normalize_pos(_current_pos), normalize_pos(_current_pos));
             }
         }
         else
         {
             // Use our own timer
-            _current_time = timer::get_microseconds(timer::monotonic);
+            _master_time_current = timer::get_microseconds(timer::monotonic) - _master_time_start
+                + _master_time_pos;
         }
 
-        int64_t time_to_next_frame = (_next_frame_pos - _sync_point_pos) - (_current_time - _sync_point_time) - _sync_point_av_offset;
-        if (time_to_next_frame <= 0 || _benchmark)
+        if (_master_time_current >= _video_pos || _benchmark)
         {
             // Output current video frame
             _drop_next_frame = false;
-            if (-time_to_next_frame > _input->video_frame_duration() * 75 / 100 && !_benchmark)
+            if (_master_time_current - _video_pos > _input->video_frame_duration() * 75 / 100 && !_benchmark)
             {
-                msg::wrn("video: delay %g seconds; dropping next frame", (-time_to_next_frame) / 1e6f);
+                msg::wrn("video: delay %g seconds; dropping next frame", (_master_time_current - _video_pos) / 1e6f);
                 _drop_next_frame = true;
             }
             if (!_previous_frame_dropped)
@@ -425,16 +474,16 @@ void player::run_step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool
                 if (_benchmark)
                 {
                     _frames_shown++;
-                    if (_frames_shown % 100 == 0) //show fps each 100 frames
+                    if (_frames_shown == 100)   //show fps each 100 frames
                     {
-                        msg::inf("fps: %.2f", 100.0f / ((_current_time - _fps_mark_time) / 1e6));
-                        _fps_mark_time = _current_time;
+                        int64_t now = timer::get_microseconds(timer::monotonic);
+                        msg::inf("fps: %.2f", static_cast<float>(_frames_shown) / ((now - _fps_mark_time) / 1e6f));
+                        _fps_mark_time = now;
                         _frames_shown = 0;
                     }
                 }
             }
             _need_frame = true;
-            _current_pos = _next_frame_pos;
             _previous_frame_dropped = _drop_next_frame;
         }
         *more_steps = true;
@@ -442,14 +491,22 @@ void player::run_step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool
     }
 }
 
-void player::get_video_frame(enum decoder::video_frame_format fmt)
+void player::get_video_frame()
 {
-    _input->get_video_frame(fmt, _l_data, _l_line_size, _r_data, _r_line_size);
+    _input->prepare_video_frame();
 }
 
 void player::prepare_video_frame(video_output *vo)
 {
-    vo->prepare(_l_data, _l_line_size, _r_data, _r_line_size);
+    for (int i = 0; i < (_input->video_is_mono() ? 1 : 2); i++)
+    {
+        for (int j = 0; j < decoder::video_format_planes(_input->video_format()); j++)
+        {
+            void *buf = vo->prepare_start(i, j);
+            _input->get_video_frame(i, j, buf);
+            vo->prepare_finish(i, j);
+        }
+    }
 }
 
 void player::release_video_frame()
@@ -463,7 +520,7 @@ void player::open(const player_init_data &init_data)
     set_benchmark(init_data.benchmark);
     reset_playstate();
     create_decoders(init_data.filenames);
-    create_input(init_data.input_mode);
+    create_input(init_data.input_mode, init_data.audio_stream);
     create_audio_output();
     create_video_output();
     _video_state = init_data.video_state;
@@ -526,7 +583,7 @@ void player::run()
         }
         if (prep_frame)
         {
-            get_video_frame(_video_output->frame_format());
+            get_video_frame();
             prepare_video_frame(_video_output);
             release_video_frame();
         }
@@ -591,6 +648,30 @@ void player::receive_cmd(const command &cmd)
         _seek_request = cmd.param * 1e6f;
         /* notify when request is fulfilled */
         break;
+    case command::set_pos:
+        _set_pos_request = cmd.param;
+        /* notify when request is fulfilled */
+        break;
+    case command::adjust_parallax:
+        oldval = _video_state.parallax;
+        _video_state.parallax = std::max(std::min(_video_state.parallax + cmd.param, 1.0f), -1.0f);
+        notify(notification::parallax, oldval, _video_state.parallax);
+        break;
+    case command::set_parallax:
+        oldval = _video_state.parallax;
+        _video_state.parallax = std::max(std::min(cmd.param, 1.0f), -1.0f);
+        notify(notification::parallax, oldval, _video_state.parallax);
+        break;
+    case command::adjust_ghostbust:
+        oldval = _video_state.ghostbust;
+        _video_state.ghostbust = std::max(std::min(_video_state.ghostbust + cmd.param, 1.0f), 0.0f);
+        notify(notification::ghostbust, oldval, _video_state.ghostbust);
+        break;
+    case command::set_ghostbust:
+        oldval = _video_state.ghostbust;
+        _video_state.ghostbust = std::max(std::min(cmd.param, 1.0f), 0.0f);
+        notify(notification::ghostbust, oldval, _video_state.ghostbust);
+        break;
     }
 }
 
