@@ -1,9 +1,9 @@
 /*
  * This file is part of bino, a 3D video player.
  *
- * Copyright (C) 2010
+ * Copyright (C) 2010-2011
  * Martin Lambers <marlam@marlam.de>
- * Lion-Simba <lion-simba@pridelands.ru>
+ * Alexey Osipov <lion-simba@pridelands.ru>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,6 +39,7 @@ public:
     msg::level_t log_level;
     bool benchmark;
     std::vector<std::string> filenames;
+    int audio_stream;
     enum input::mode input_mode;
     enum video_output::mode video_mode;
     video_output_state video_state;
@@ -76,25 +77,27 @@ private:
     bool _quit_request;
     bool _pause_request;
     int64_t _seek_request;
+    float _set_pos_request;
 
-    uint8_t *_l_data[3], *_r_data[3];
-    size_t _l_line_size[3], _r_line_size[3];
     void *_audio_data;
     size_t _required_audio_data_size;
     int64_t _pause_start;
     // Audio / video timing, relative to a synchronization point.
     // The master time is the audio time, or external time if there is no audio.
     // All times are in microseconds.
-    int64_t _sync_point_pos;             // input position at last sync point
-    int64_t _sync_point_time;            // master time at last sync point
-    int64_t _sync_point_av_offset;       // audio/video offset at last sync point
-    int64_t _current_pos;                // current input position
-    int64_t _current_time;               // current master time
-    int64_t _next_frame_pos;             // presentation time of next video frame
+    int64_t _start_pos;                 // initial input position
+    int64_t _current_pos;               // current input position
+    int64_t _video_pos;                 // presentation time of current video frame
+    int64_t _audio_pos;                 // presentation time of current audio data block
+    int64_t _master_time_start;         // master time offset
+    int64_t _master_time_current;       // current master time
+    int64_t _master_time_pos;           // input position at master time start
 
     int _frames_shown;                   // frames shown since last _sync_point_time update
     int64_t _fps_mark_time;              // time when _frames_shown was reset to zero
 
+    float normalize_pos(int64_t pos);   // transform a position into [0,1]
+
 protected:
     void set_benchmark(bool benchmark)
     {
@@ -102,8 +105,7 @@ protected:
     }
     void reset_playstate();
     void create_decoders(const std::vector<std::string> &filenames);
-    void create_input(enum input::mode input_mode);
-    void get_input_info(int *w, int *h, float *ar, enum decoder::video_frame_format *fmt);
+    void create_input(enum input::mode input_mode, int audio_stream);
     void create_audio_output();
     void create_video_output();
     void set_video_output(video_output *vo)
@@ -115,7 +117,7 @@ protected:
     void make_master();
     void run_step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool *drop_frame, bool *display_frame);
     void seek(int64_t seek_to);
-    void get_video_frame(enum decoder::video_frame_format fmt);
+    void get_video_frame();
     void prepare_video_frame(video_output *vo);
     void release_video_frame();
     input *get_input() { return _input; }
