@@ -1,7 +1,7 @@
 /*
  * This file is part of bino, a 3D video player.
  *
- * Copyright (C) 2010
+ * Copyright (C) 2010-2011
  * Martin Lambers <marlam@marlam.de>
  * Stefan Eilemann <eile@eyescale.ch>
  *
@@ -65,7 +65,6 @@ class player_eq_node : public player
 {
 private:
     bool _is_master;
-    enum decoder::video_frame_format _fmt;
 
 public:
     player_eq_node() : player(player::slave), _is_master(false)
@@ -80,16 +79,19 @@ public:
 
     bool eq_init(const player_init_data &init_data,
             int *src_width, int *src_height, float *src_aspect_ratio,
-            enum decoder::video_frame_format *src_preferred_frame_format)
+            int *src_frame_format, bool *src_is_mono)
     {
         try
         {
             set_benchmark(init_data.benchmark);
             reset_playstate();
             create_decoders(init_data.filenames);
-            create_input(init_data.input_mode);
-            get_input_info(src_width, src_height, src_aspect_ratio, src_preferred_frame_format);
-            _fmt = *src_preferred_frame_format;
+            create_input(init_data.input_mode, init_data.audio_stream);
+            *src_width = get_input()->video_width();
+            *src_height = get_input()->video_height();
+            *src_aspect_ratio = get_input()->video_aspect_ratio();
+            *src_frame_format = get_input()->video_format();
+            *src_is_mono = get_input()->video_is_mono();
             if (_is_master)
             {
                 create_audio_output();
@@ -134,7 +136,7 @@ public:
 
     void eq_get_frame()
     {
-        get_video_frame(_fmt);
+        get_video_frame();
     }
 
     void eq_release_frame()
@@ -156,16 +158,16 @@ public:
     {
     }
 
-    void eq_display(bool toggle_swap_eyes, float x, float y, float w, float h)
+    void eq_display(bool toggle_swap_eyes, float x, float y, float w, float h, const int viewport[4])
     {
-        video_output_opengl::display(toggle_swap_eyes, x, y, w, h);
+        video_output_opengl::display(toggle_swap_eyes, x, y, w, h, viewport);
     }
 
     void eq_initialize(int src_width, int src_height, float src_aspect_ratio,
-            enum decoder::video_frame_format src_preferred_frame_format)
+            int src_frame_format, bool src_is_mono)
     {
         set_mode(mono_left);    // to display the right view, we can toggle the swap_eyes flag
-        set_source_info(src_width, src_height, src_aspect_ratio, src_preferred_frame_format);
+        set_source_info(src_width, src_height, src_aspect_ratio, src_frame_format, src_is_mono);
         initialize();
     }
 
@@ -189,7 +191,8 @@ public:
     virtual int screen_pos_y() { return 0; }
     virtual void receive_notification(const notification &) {}
     virtual bool supports_stereo() { return false; }
-    virtual void open(enum decoder::video_frame_format, int, int, float, int, const video_output_state&, unsigned int, int, int) {}
+    virtual void open(int, bool, int, int, float, int,
+            const video_output_state&, unsigned int, int, int) {}
     virtual void activate() {}
     virtual void process_events() {}
     virtual void close() {}
@@ -234,12 +237,18 @@ protected:
         s11n::save(oss, frame_data_id);
         s11n::save(oss, static_cast<int>(init_data.log_level));
         s11n::save(oss, init_data.filenames);
+        s11n::save(oss, init_data.audio_stream);
         s11n::save(oss, static_cast<int>(init_data.input_mode));
         s11n::save(oss, static_cast<int>(init_data.video_mode));
         s11n::save(oss, init_data.video_state.contrast);
         s11n::save(oss, init_data.video_state.brightness);
         s11n::save(oss, init_data.video_state.hue);
         s11n::save(oss, init_data.video_state.saturation);
+        s11n::save(oss, init_data.video_state.parallax);
+        s11n::save(oss, init_data.video_state.crosstalk_r);
+        s11n::save(oss, init_data.video_state.crosstalk_g);
+        s11n::save(oss, init_data.video_state.crosstalk_b);
+        s11n::save(oss, init_data.video_state.ghostbust);
         s11n::save(oss, init_data.video_state.fullscreen);
         s11n::save(oss, init_data.video_state.swap_eyes);
         s11n::save(oss, init_data.video_flags);
@@ -262,6 +271,7 @@ protected:
         s11n::load(iss, x);
         init_data.log_level = static_cast<msg::level_t>(x);
         s11n::load(iss, init_data.filenames);
+        s11n::load(iss, init_data.audio_stream);
         s11n::load(iss, x);
         init_data.input_mode = static_cast<enum input::mode>(x);
         s11n::load(iss, x);
@@ -270,6 +280,11 @@ protected:
         s11n::load(iss, init_data.video_state.brightness);
         s11n::load(iss, init_data.video_state.hue);
         s11n::load(iss, init_data.video_state.saturation);
+        s11n::load(iss, init_data.video_state.parallax);
+        s11n::load(iss, init_data.video_state.crosstalk_r);
+        s11n::load(iss, init_data.video_state.crosstalk_g);
+        s11n::load(iss, init_data.video_state.crosstalk_b);
+        s11n::load(iss, init_data.video_state.ghostbust);
         s11n::load(iss, init_data.video_state.fullscreen);
         s11n::load(iss, init_data.video_state.swap_eyes);
         s11n::load(iss, init_data.video_flags);
@@ -315,6 +330,11 @@ protected:
         s11n::save(oss, video_state.brightness);
         s11n::save(oss, video_state.hue);
         s11n::save(oss, video_state.saturation);
+        s11n::save(oss, video_state.parallax);
+        s11n::save(oss, video_state.crosstalk_r);
+        s11n::save(oss, video_state.crosstalk_g);
+        s11n::save(oss, video_state.crosstalk_b);
+        s11n::save(oss, video_state.ghostbust);
         s11n::save(oss, video_state.fullscreen);
         s11n::save(oss, video_state.swap_eyes);
         s11n::save(oss, seek_to);
@@ -333,6 +353,11 @@ protected:
         s11n::load(iss, video_state.brightness);
         s11n::load(iss, video_state.hue);
         s11n::load(iss, video_state.saturation);
+        s11n::load(iss, video_state.parallax);
+        s11n::load(iss, video_state.crosstalk_r);
+        s11n::load(iss, video_state.crosstalk_g);
+        s11n::load(iss, video_state.crosstalk_b);
+        s11n::load(iss, video_state.ghostbust);
         s11n::load(iss, video_state.fullscreen);
         s11n::load(iss, video_state.swap_eyes);
         s11n::load(iss, seek_to);
@@ -359,14 +384,15 @@ public:
     // Source video properties:
     int src_width, src_height;
     float src_aspect_ratio;
-    enum decoder::video_frame_format src_preferred_frame_format;
+    int src_frame_format;
+    bool src_is_mono;
 
 public:
     eq_config(eq::ServerPtr parent)
         : eq::Config(parent), _is_master_config(false), _eq_init_data(), _eq_frame_data(),
         _player(), _controller(false),
         src_width(-1), src_height(-1), src_aspect_ratio(0.0f),
-        src_preferred_frame_format(decoder::frame_format_yuv420p)
+        src_frame_format(0), src_is_mono(true)
     {
     }
 
@@ -387,7 +413,8 @@ public:
         _eq_frame_data.video_state = _eq_init_data.init_data.video_state;
         // Initialize master player
         _player.eq_make_master();
-        if (!_player.eq_init(init_data, &src_width, &src_height, &src_aspect_ratio, &src_preferred_frame_format))
+        if (!_player.eq_init(init_data, &src_width, &src_height, &src_aspect_ratio,
+                    &src_frame_format, &src_is_mono))
         {
             return false;
         }
@@ -522,6 +549,18 @@ public:
             case '8':
                 _controller.send_cmd(command::adjust_saturation, +0.05f);
                 break;
+            case '<':
+                _controller.send_cmd(command::adjust_parallax, -0.01f);
+                break;
+            case '>':
+                _controller.send_cmd(command::adjust_parallax, +0.01f);
+                break;
+            case '(':
+                _controller.send_cmd(command::adjust_ghostbust, -0.01f);
+                break;
+            case ')':
+                _controller.send_cmd(command::adjust_ghostbust, +0.01f);
+                break;
             case eq::KC_LEFT:
                 _controller.send_cmd(command::seek, -10.0f);
                 break;
@@ -636,13 +675,14 @@ public:
     eq_frame_data frame_data;
     int src_width, src_height;
     float src_aspect_ratio;
-    enum decoder::video_frame_format src_preferred_frame_format;
+    int src_frame_format;
+    bool src_is_mono;
 
     eq_node(eq::Config *parent)
         : eq::Node(parent), _is_app_node(false),
         _player(), init_data(), frame_data(),
         src_width(-1), src_height(-1), src_aspect_ratio(-1.0f),
-        src_preferred_frame_format(decoder::frame_format_yuv420p)
+        src_frame_format(0), src_is_mono(true)
     {
     }
 
@@ -675,7 +715,8 @@ protected:
         // Create decoders and input
         if (!_is_app_node)
         {
-            if (!_player.eq_init(init_data.init_data, &src_width, &src_height, &src_aspect_ratio, &src_preferred_frame_format))
+            if (!_player.eq_init(init_data.init_data, &src_width, &src_height, &src_aspect_ratio,
+                        &src_frame_format, &src_is_mono))
             {
                 return false;
             }
@@ -685,7 +726,8 @@ protected:
             src_width = config->src_width;
             src_height = config->src_height;
             src_aspect_ratio = config->src_aspect_ratio;
-            src_preferred_frame_format = config->src_preferred_frame_format;
+            src_frame_format = config->src_frame_format;
+            src_is_mono = config->src_is_mono;
         }
         msg::dbg(HERE);
         return true;
@@ -764,13 +806,8 @@ protected:
     }
 
 public:
-    void prep_frame(video_output *vo, enum decoder::video_frame_format fmt)
+    void prep_frame(video_output *vo)
     {
-        if (fmt != src_preferred_frame_format)
-        {
-            msg::err("cannot provide video in requested frame format");
-            abort();
-        }
         if (_is_app_node)
         {
             eq_config *config = static_cast<eq_config *>(getConfig());
@@ -809,9 +846,9 @@ public:
     {
     }
 
-    void display(bool toggle_swap_eyes, float x, float y, float w, float h)
+    void display(bool toggle_swap_eyes, float x, float y, float w, float h, const int viewport[4])
     {
-        _video_output.eq_display(toggle_swap_eyes, x, y, w, h);
+        _video_output.eq_display(toggle_swap_eyes, x, y, w, h, viewport);
     }
 
 protected:
@@ -834,7 +871,7 @@ protected:
 
         eq_node *node = static_cast<eq_node *>(getNode());
         _video_output.eq_initialize(node->src_width, node->src_height,
-                node->src_aspect_ratio, node->src_preferred_frame_format);
+                node->src_aspect_ratio, node->src_frame_format, node->src_is_mono);
 
         msg::dbg(HERE);
         return true;
@@ -857,7 +894,7 @@ protected:
         if (node->frame_data.prep_frame)
         {
             makeCurrent();      // XXX Is this necessary?
-            node->prep_frame(&_video_output, _video_output.frame_format());
+            node->prep_frame(&_video_output);
         }
         if (node->frame_data.display_frame)
         {
@@ -926,7 +963,9 @@ protected:
         // Display
         eq_window *window = static_cast<eq_window *>(getWindow());
         bool toggle_swap_eyes = (getEye() == eq::EYE_RIGHT);
-        window->display(toggle_swap_eyes, quad_x, quad_y, quad_w, quad_h);
+        GLint viewport[4];
+        glGetIntegerv(GL_VIEWPORT, viewport);
+        window->display(toggle_swap_eyes, quad_x, quad_y, quad_w, quad_h, viewport);
     }
 };
 
