@@ -1,7 +1,9 @@
 /*
  * This file is part of bino, a 3D video player.
  *
- * Copyright (C) 2010  Martin Lambers <marlam@marlam.de>
+ * Copyright (C) 2010-2011
+ * Martin Lambers <marlam@marlam.de>
+ * Frédéric Devernay <Frederic.Devernay@inrialpes.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +22,7 @@
 #include "config.h"
 
 #include <QCoreApplication>
+#include <QApplication>
 #include <QMainWindow>
 #include <QGridLayout>
 #include <QCloseEvent>
@@ -37,6 +40,10 @@
 #include <QFile>
 #include <QByteArray>
 #include <QCryptographicHash>
+#include <QUrl>
+#include <QDesktopServices>
+#include <QDoubleSpinBox>
+#include <QSpinBox>
 
 #include "player_qt.h"
 #include "qt_app.h"
@@ -60,7 +67,7 @@ void player_qt_internal::open(const player_init_data &init_data)
     reset_playstate();
     set_benchmark(init_data.benchmark);
     create_decoders(init_data.filenames);
-    create_input(init_data.input_mode);
+    create_input(init_data.input_mode, init_data.audio_stream);
     create_audio_output();
     set_video_output(new video_output_opengl_qt(_container_widget));
     video_state() = init_data.video_state;
@@ -102,7 +109,7 @@ bool player_qt_internal::playloop_step()
     }
     if (prep_frame)
     {
-        get_video_frame(get_video_output()->frame_format());
+        get_video_frame();
         prepare_video_frame(get_video_output());
         release_video_frame();
     }
@@ -148,8 +155,16 @@ in_out_widget::in_out_widget(QSettings *settings, QWidget *parent)
     _input_combobox->addItem("Even/odd rows");
     connect(_input_combobox, SIGNAL(currentIndexChanged(int)), this, SLOT(input_changed()));
     layout0->addWidget(_input_combobox, 0, 1);
+    layout0->setColumnStretch(1, 1);
+    QLabel *audio_label = new QLabel("Audio:");
+    layout0->addWidget(audio_label, 0, 2);
+    _audio_spinbox = new QSpinBox();
+    _audio_spinbox->setRange(1, 999);
+    _audio_spinbox->setValue(1);
+    layout0->addWidget(_audio_spinbox, 0, 3);
     QLabel *output_label = new QLabel("Output:");
-    layout0->addWidget(output_label, 1, 0);
+    QGridLayout *layout1 = new QGridLayout;
+    layout1->addWidget(output_label, 1, 0);
     _output_combobox = new QComboBox(this);
     _output_combobox->addItem("Left view");
     _output_combobox->addItem("Right view");
@@ -165,30 +180,56 @@ in_out_widget::in_out_widget(QSettings *settings, QWidget *parent)
     _output_combobox->addItem("Red/cyan glasses, full-color method");
     _output_combobox->addItem("Red/cyan glasses, half-color method");
     _output_combobox->addItem("OpenGL stereo");
-    layout0->addWidget(_output_combobox, 1, 1);
-    layout0->setColumnStretch(1, 1);
-    QGridLayout *layout1 = new QGridLayout;
+    layout1->addWidget(_output_combobox, 1, 1);
+    layout1->setColumnStretch(1, 1);
+    QGridLayout *layout2 = new QGridLayout;
     _swap_eyes_button = new QPushButton("Swap eyes");
     _swap_eyes_button->setCheckable(true);
     connect(_swap_eyes_button, SIGNAL(toggled(bool)), this, SLOT(swap_eyes_changed()));
-    layout1->addWidget(_swap_eyes_button, 0, 4);
+    layout2->addWidget(_swap_eyes_button, 0, 0, 1, 2);
     _fullscreen_button = new QPushButton("Fullscreen");
     connect(_fullscreen_button, SIGNAL(pressed()), this, SLOT(fullscreen_pressed()));
-    layout1->addWidget(_fullscreen_button, 0, 5);
+    layout2->addWidget(_fullscreen_button, 0, 2, 1, 2);
     _center_button = new QPushButton("Center");
     connect(_center_button, SIGNAL(pressed()), this, SLOT(center_pressed()));
-    layout1->addWidget(_center_button, 0, 6);
-    layout1->setRowStretch(0, 1);
+    layout2->addWidget(_center_button, 0, 4, 1, 2);
+    _parallax_label = new QLabel("Parallax:");
+    layout2->addWidget(_parallax_label, 0, 6, 1, 1);
+    _parallax_spinbox = new QDoubleSpinBox();
+    _parallax_spinbox->setRange(-1.0, +1.0);
+    _parallax_spinbox->setValue(0);
+    _parallax_spinbox->setDecimals(2);
+    _parallax_spinbox->setSingleStep(0.01);
+    connect(_parallax_spinbox, SIGNAL(valueChanged(double)), this, SLOT(parallax_changed()));
+    layout2->addWidget(_parallax_spinbox, 0, 7, 1, 1);
+    _ghostbust_label = new QLabel("Ghostbusting:");
+    layout2->addWidget(_ghostbust_label, 0, 8, 1, 1);
+    _ghostbust_spinbox = new QSpinBox();
+    _ghostbust_spinbox->setRange(0, 100);
+    _ghostbust_spinbox->setValue(0);
+    connect(_ghostbust_spinbox, SIGNAL(valueChanged(int)), this, SLOT(ghostbust_changed()));
+    layout2->addWidget(_ghostbust_spinbox, 0, 9, 1, 1);
+    layout2->setRowStretch(0, 1);
     QGridLayout *layout = new QGridLayout;
     layout->addLayout(layout0, 0, 0);
     layout->addLayout(layout1, 1, 0);
+    layout->addLayout(layout2, 2, 0);
     setLayout(layout);
 
+    // Align the input and output labels
+    output_label->setMinimumSize(output_label->minimumSizeHint());
+    input_label->setMinimumSize(output_label->minimumSizeHint());
+
     _input_combobox->setEnabled(false);
+    _audio_spinbox->setEnabled(false);
     _output_combobox->setEnabled(false);
     _swap_eyes_button->setEnabled(false);
     _fullscreen_button->setEnabled(false);
     _center_button->setEnabled(false);
+    _parallax_label->setEnabled(false);
+    _parallax_spinbox->setEnabled(false);
+    _ghostbust_label->setEnabled(false);
+    _ghostbust_spinbox->setEnabled(false);
 }
 
 in_out_widget::~in_out_widget()
@@ -312,12 +353,31 @@ void in_out_widget::center_pressed()
     send_cmd(command::center);
 }
 
+void in_out_widget::parallax_changed()
+{
+    if (!_lock)
+    {
+        send_cmd(command::set_parallax, _parallax_spinbox->value());
+    }
+}
+
+void in_out_widget::ghostbust_changed()
+{
+    if (!_lock)
+    {
+        send_cmd(command::set_ghostbust, _ghostbust_spinbox->value() / 100.0f);
+    }
+}
+
 void in_out_widget::update(const player_init_data &init_data, bool have_valid_input, bool playing)
 {
     set_input(init_data.input_mode);
     set_output(init_data.video_mode);
     _lock = true;
+    _audio_spinbox->setValue(init_data.audio_stream + 1);
     _swap_eyes_button->setChecked(init_data.video_state.swap_eyes);
+    _parallax_spinbox->setValue(init_data.video_state.parallax);
+    _ghostbust_spinbox->setValue(qRound(init_data.video_state.ghostbust * 100.0f));
     _lock = false;
     if (have_valid_input)
     {
@@ -326,10 +386,15 @@ void in_out_widget::update(const player_init_data &init_data, bool have_valid_in
     else
     {
         _input_combobox->setEnabled(false);
+        _audio_spinbox->setEnabled(false);
         _output_combobox->setEnabled(false);
         _swap_eyes_button->setEnabled(false);
         _fullscreen_button->setEnabled(false);
         _center_button->setEnabled(false);
+        _parallax_label->setEnabled(false);
+        _parallax_spinbox->setEnabled(false);
+        _ghostbust_label->setEnabled(false);
+        _ghostbust_spinbox->setEnabled(false);
     }
 }
 
@@ -355,6 +420,11 @@ enum input::mode in_out_widget::input_mode()
     }
 }
 
+int in_out_widget::audio_stream()
+{
+    return _audio_spinbox->value() - 1;
+}
+
 enum video_output::mode in_out_widget::video_mode()
 {
     switch (_output_combobox->currentIndex())
@@ -397,16 +467,45 @@ void in_out_widget::receive_notification(const notification &note)
     {
     case notification::play:
         _input_combobox->setEnabled(!note.current.flag);
+        _audio_spinbox->setEnabled(!note.current.flag);
         _output_combobox->setEnabled(!note.current.flag);
         _swap_eyes_button->setEnabled(note.current.flag);
         _fullscreen_button->setEnabled(note.current.flag);
         _center_button->setEnabled(note.current.flag);
+        {
+            bool parallax = (note.current.flag && input_mode() != input::mono);
+            _parallax_label->setEnabled(parallax);
+            _parallax_spinbox->setEnabled(parallax);
+        }
+        {
+            bool ghostbust = (note.current.flag
+                    && video_mode() != video_output::anaglyph_red_cyan_dubois
+                    && video_mode() != video_output::anaglyph_red_cyan_monochrome
+                    && video_mode() != video_output::anaglyph_red_cyan_full_color
+                    && video_mode() != video_output::anaglyph_red_cyan_half_color);
+            _ghostbust_label->setEnabled(ghostbust);
+            _ghostbust_spinbox->setEnabled(ghostbust);
+        }
         break;
     case notification::swap_eyes:
         _lock = true;
         _swap_eyes_button->setChecked(note.current.flag);
         _lock = false;
         break;
+    case notification::parallax:
+        _lock = true;
+        if (_parallax_spinbox->isEnabled())
+        {
+            _parallax_spinbox->setValue(note.current.value);
+        }
+        _lock = false;
+    case notification::ghostbust:
+        _lock = true;
+        if (_ghostbust_spinbox->isEnabled())
+        {
+            _ghostbust_spinbox->setValue(qRound(note.current.value * 100.0f));
+        }
+        _lock = false;
     default:
         break;
     }
@@ -414,37 +513,42 @@ void in_out_widget::receive_notification(const notification &note)
 
 
 controls_widget::controls_widget(QSettings *settings, QWidget *parent)
-    : QWidget(parent), _settings(settings), _playing(false)
+    : QWidget(parent), _lock(false), _settings(settings), _playing(false)
 {
     QGridLayout *layout = new QGridLayout;
+    _seek_slider = new QSlider(Qt::Horizontal);
+    _seek_slider->setRange(0, 2000);
+    _seek_slider->setTracking(false);
+    connect(_seek_slider, SIGNAL(valueChanged(int)), this, SLOT(seek_slider_changed()));
+    layout->addWidget(_seek_slider, 0, 0, 1, 10);
     _play_button = new QPushButton(QIcon(":icons/play.png"), "");
     connect(_play_button, SIGNAL(pressed()), this, SLOT(play_pressed()));
-    layout->addWidget(_play_button, 0, 0);
+    layout->addWidget(_play_button, 1, 0);
     _pause_button = new QPushButton(QIcon(":icons/pause.png"), "");
     connect(_pause_button, SIGNAL(pressed()), this, SLOT(pause_pressed()));
-    layout->addWidget(_pause_button, 0, 1);
+    layout->addWidget(_pause_button, 1, 1);
     _stop_button = new QPushButton(QIcon(":icons/stop.png"), "");
     connect(_stop_button, SIGNAL(pressed()), this, SLOT(stop_pressed()));
-    layout->addWidget(_stop_button, 0, 2);
-    layout->addWidget(new QWidget, 0, 3);
+    layout->addWidget(_stop_button, 1, 2);
+    layout->addWidget(new QWidget, 1, 3);
     _bbb_button = new QPushButton(QIcon(":icons/bbb.png"), "");
     connect(_bbb_button, SIGNAL(pressed()), this, SLOT(bbb_pressed()));
-    layout->addWidget(_bbb_button, 0, 4);
+    layout->addWidget(_bbb_button, 1, 4);
     _bb_button = new QPushButton(QIcon(":icons/bb.png"), "");
     connect(_bb_button, SIGNAL(pressed()), this, SLOT(bb_pressed()));
-    layout->addWidget(_bb_button, 0, 5);
+    layout->addWidget(_bb_button, 1, 5);
     _b_button = new QPushButton(QIcon(":icons/b.png"), "");
     connect(_b_button, SIGNAL(pressed()), this, SLOT(b_pressed()));
-    layout->addWidget(_b_button, 0, 6);
+    layout->addWidget(_b_button, 1, 6);
     _f_button = new QPushButton(QIcon(":icons/f.png"), "");
     connect(_f_button, SIGNAL(pressed()), this, SLOT(f_pressed()));
-    layout->addWidget(_f_button, 0, 7);
+    layout->addWidget(_f_button, 1, 7);
     _ff_button = new QPushButton(QIcon(":icons/ff.png"), "");
     connect(_ff_button, SIGNAL(pressed()), this, SLOT(ff_pressed()));
-    layout->addWidget(_ff_button, 0, 8);
+    layout->addWidget(_ff_button, 1, 8);
     _fff_button = new QPushButton(QIcon(":icons/fff.png"), "");
     connect(_fff_button, SIGNAL(pressed()), this, SLOT(fff_pressed()));
-    layout->addWidget(_fff_button, 0, 9);
+    layout->addWidget(_fff_button, 1, 9);
     layout->setRowStretch(0, 0);
     layout->setColumnStretch(3, 1);
     setLayout(layout);
@@ -458,6 +562,7 @@ controls_widget::controls_widget(QSettings *settings, QWidget *parent)
     _f_button->setEnabled(false);
     _ff_button->setEnabled(false);
     _fff_button->setEnabled(false);
+    _seek_slider->setEnabled(false);
 }
 
 controls_widget::~controls_widget()
@@ -516,6 +621,14 @@ void controls_widget::fff_pressed()
     send_cmd(command::seek, +600.0f);
 }
 
+void controls_widget::seek_slider_changed()
+{
+    if (!_lock)
+    {
+        send_cmd(command::set_pos, static_cast<float>(_seek_slider->value()) / 2000.0f);
+    }
+}
+
 void controls_widget::update(const player_init_data &, bool have_valid_input, bool playing)
 {
     if (have_valid_input)
@@ -534,6 +647,8 @@ void controls_widget::update(const player_init_data &, bool have_valid_input, bo
         _f_button->setEnabled(false);
         _ff_button->setEnabled(false);
         _fff_button->setEnabled(false);
+        _seek_slider->setEnabled(false);
+        _seek_slider->setValue(0);
     }
 }
 
@@ -552,11 +667,23 @@ void controls_widget::receive_notification(const notification &note)
         _f_button->setEnabled(note.current.flag);
         _ff_button->setEnabled(note.current.flag);
         _fff_button->setEnabled(note.current.flag);
+        _seek_slider->setEnabled(note.current.flag);
+        if (!note.current.flag)
+        {
+            _seek_slider->setValue(0);
+        }
         break;
     case notification::pause:
         _play_button->setEnabled(note.current.flag);
         _pause_button->setEnabled(!note.current.flag);
         break;
+    case notification::pos:
+        _lock = true;
+        if (!_seek_slider->isSliderDown())
+        {
+            _seek_slider->setValue(qRound(note.current.value * 2000.0f));
+        }
+        _lock = false;
     default:
         break;
     }
@@ -570,6 +697,13 @@ main_window::main_window(QSettings *settings, const player_init_data &init_data)
     setWindowTitle(PACKAGE_NAME);
     setWindowIcon(QIcon(":icons/appicon.png"));
 
+    // Load preferences
+    _settings->beginGroup("Session");
+    _init_data.video_state.crosstalk_r = _settings->value("crosstalk_r", QString("0")).toFloat();
+    _init_data.video_state.crosstalk_g = _settings->value("crosstalk_g", QString("0")).toFloat();
+    _init_data.video_state.crosstalk_b = _settings->value("crosstalk_b", QString("0")).toFloat();
+    _settings->endGroup();
+
     // Central widget
     QWidget *central_widget = new QWidget(this);
     QGridLayout *layout = new QGridLayout();
@@ -603,16 +737,30 @@ main_window::main_window(QSettings *settings, const player_init_data &init_data)
 #endif
     connect(file_quit_act, SIGNAL(triggered()), this, SLOT(close()));
     file_menu->addAction(file_quit_act);
+    QMenu *preferences_menu = menuBar()->addMenu(tr("&Preferences"));
+    QAction *preferences_crosstalk_act = new QAction(tr("&Crosstalk..."), this);
+    connect(preferences_crosstalk_act, SIGNAL(triggered()), this, SLOT(preferences_crosstalk()));
+    preferences_menu->addAction(preferences_crosstalk_act);
     QMenu *help_menu = menuBar()->addMenu(tr("&Help"));
+    QAction *help_manual_act = new QAction(tr("&Manual..."), this);
+    help_manual_act->setShortcut(QKeySequence::HelpContents);
+    connect(help_manual_act, SIGNAL(triggered()), this, SLOT(help_manual()));
+    help_menu->addAction(help_manual_act);
+    QAction *help_website_act = new QAction(tr("&Website..."), this);
+    connect(help_website_act, SIGNAL(triggered()), this, SLOT(help_website()));
+    help_menu->addAction(help_website_act);
     QAction *help_keyboard_act = new QAction(tr("&Keyboard Shortcuts"), this);
-    help_keyboard_act->setShortcut(QKeySequence::HelpContents);
     connect(help_keyboard_act, SIGNAL(triggered()), this, SLOT(help_keyboard()));
     help_menu->addAction(help_keyboard_act);
     QAction *help_about_act = new QAction(tr("&About"), this);
     connect(help_about_act, SIGNAL(triggered()), this, SLOT(help_about()));
     help_menu->addAction(help_about_act);
 
+    // Handle FileOpen events
+    QApplication::instance()->installEventFilter(this);
+
     show();     // Must happen before opening initial files!
+    raise();
 
     // Player and timer
     _player = new player_qt_internal(_video_container_widget);
@@ -644,6 +792,14 @@ main_window::~main_window()
     }
 }
 
+QString main_window::current_file_hash()
+{
+    // Return SHA1 hash of the name of the current file as a hex string
+    QString name = QFileInfo(QFile::decodeName(_init_data.filenames[0].c_str())).fileName();
+    QString hash = QCryptographicHash::hash(name.toUtf8(), QCryptographicHash::Sha1).toHex();
+    return hash;
+}
+
 bool main_window::open_player()
 {
     try
@@ -670,20 +826,19 @@ void main_window::receive_notification(const notification &note)
             // current choice.
             _player->close();
             _init_data.input_mode = _in_out_widget->input_mode();
+            _init_data.audio_stream = _in_out_widget->audio_stream();
             _init_data.video_mode = _in_out_widget->video_mode();
             if (!open_player())
             {
                 _stop_request = true;
             }
-            // Remember the input mode of this video, using an SHA1 hash of its
-            // filename.
+            // Remember the input settings of this video, using an SHA1 hash of its filename.
             _settings->beginGroup("Video");
             if (_init_data.filenames.size() == 1)
             {
-                QString name = QFileInfo(QFile::decodeName(_init_data.filenames[0].c_str())).fileName();
-                QByteArray hash = QCryptographicHash::hash(name.toUtf8(), QCryptographicHash::Sha1);
-                _settings->setValue(QString(hash.toHex()), QString(input::mode_name(_init_data.input_mode).c_str()));
+                _settings->setValue(current_file_hash(), QString(input::mode_name(_init_data.input_mode).c_str()));
             }
+            _settings->setValue(current_file_hash() + "-audio-stream", QVariant(_init_data.audio_stream + 1).toString());
             _settings->endGroup();
             // Remember the 2D or 3D video output mode.
             _settings->beginGroup("Session");
@@ -710,6 +865,43 @@ void main_window::receive_notification(const notification &note)
             _player->close();
         }
     }
+    else if (note.type == notification::contrast)
+    {
+        _init_data.video_state.contrast = note.current.value;
+    }
+    else if (note.type == notification::brightness)
+    {
+        _init_data.video_state.brightness = note.current.value;
+    }
+    else if (note.type == notification::hue)
+    {
+        _init_data.video_state.hue = note.current.value;
+    }
+    else if (note.type == notification::saturation)
+    {
+        _init_data.video_state.saturation = note.current.value;
+    }
+    else if (note.type == notification::swap_eyes)
+    {
+        _init_data.video_state.swap_eyes = note.current.flag;
+        _settings->beginGroup("Video");
+        _settings->setValue(current_file_hash() + "-swap-eyes", QVariant(_init_data.video_state.swap_eyes).toString());
+        _settings->endGroup();
+    }
+    else if (note.type == notification::parallax)
+    {
+        _init_data.video_state.parallax = note.current.value;
+        _settings->beginGroup("Video");
+        _settings->setValue(current_file_hash() + "-parallax", QVariant(_init_data.video_state.parallax).toString());
+        _settings->endGroup();
+    }
+    else if (note.type == notification::ghostbust)
+    {
+        _init_data.video_state.ghostbust = note.current.value;
+        _settings->beginGroup("Video");
+        _settings->setValue(current_file_hash() + "-ghostbust", QVariant(_init_data.video_state.ghostbust).toString());
+        _settings->endGroup();
+    }
 }
 
 void main_window::moveEvent(QMoveEvent *)
@@ -719,6 +911,12 @@ void main_window::moveEvent(QMoveEvent *)
 
 void main_window::closeEvent(QCloseEvent *event)
 {
+    // Remember the preferences
+    _settings->beginGroup("Session");
+    _settings->setValue("crosstalk_r", QVariant(_init_data.video_state.crosstalk_r).toString());
+    _settings->setValue("crosstalk_g", QVariant(_init_data.video_state.crosstalk_g).toString());
+    _settings->setValue("crosstalk_b", QVariant(_init_data.video_state.crosstalk_b).toString());
+    _settings->endGroup();
     event->accept();
 }
 
@@ -756,6 +954,7 @@ void main_window::open(QStringList filenames, bool automatic)
     if (automatic)
     {
         _init_data.input_mode = input::automatic;
+        _init_data.audio_stream = 0;
         _init_data.video_mode = video_output::automatic;
     }
     if (open_player())
@@ -763,16 +962,18 @@ void main_window::open(QStringList filenames, bool automatic)
         _settings->beginGroup("Video");
         if (_init_data.filenames.size() == 1)
         {
-            QString name = QFileInfo(QFile::decodeName(_init_data.filenames[0].c_str())).fileName();
-            QByteArray hash = QCryptographicHash::hash(name.toUtf8(), QCryptographicHash::Sha1);
             QString fallback_mode_name = QString(input::mode_name(_player->input_mode()).c_str());
-            QString mode_name = _settings->value(QString(hash.toHex()), fallback_mode_name).toString();
+            QString mode_name = _settings->value(current_file_hash(), fallback_mode_name).toString();
             _init_data.input_mode = input::mode_from_name(mode_name.toStdString());
         }
         else
         {
             _init_data.input_mode = _player->input_mode();
         }
+        _init_data.audio_stream = QVariant(_settings->value(current_file_hash() + "-audio-stream", QString("1")).toString()).toInt() - 1;
+        _init_data.video_state.swap_eyes = QVariant(_settings->value(current_file_hash() + "-swap-eyes", QString("false")).toString()).toBool();
+        _init_data.video_state.parallax = QVariant(_settings->value(current_file_hash() + "-parallax", QString("0")).toString()).toFloat();
+        _init_data.video_state.ghostbust = QVariant(_settings->value(current_file_hash() + "-ghostbust", QString("0")).toString()).toFloat();
         _settings->endGroup();
         _settings->beginGroup("Session");
         if (automatic)
@@ -861,6 +1062,77 @@ void main_window::file_open_url()
     }
 }
 
+void main_window::preferences_crosstalk()
+{
+    QDialog *dialog = new QDialog(this);
+    dialog->setModal(true);
+    dialog->setWindowTitle("Set crosstalk levels");
+    QGridLayout *layout = new QGridLayout;
+    QLabel *rtfm_label = new QLabel(
+            "<p>Please read the manual to find out<br>"
+            "how to measure the crosstalk levels<br>"
+            "of your display.</p>");
+    layout->addWidget(rtfm_label, 0, 0, 1, 6);
+    QLabel *red_label = new QLabel("Red:");
+    layout->addWidget(red_label, 1, 0, 1, 2);
+    QSpinBox *red_spinbox = new QSpinBox();
+    red_spinbox->setRange(0, 100);
+    red_spinbox->setValue(_init_data.video_state.crosstalk_r * 100.0f);
+    layout->addWidget(red_spinbox, 1, 2, 1, 4);
+    QLabel *green_label = new QLabel("Green:");
+    layout->addWidget(green_label, 2, 0, 1, 3);
+    QSpinBox *green_spinbox = new QSpinBox();
+    green_spinbox->setRange(0, 100);
+    green_spinbox->setValue(_init_data.video_state.crosstalk_g * 100.0f);
+    layout->addWidget(green_spinbox, 2, 2, 1, 4);
+    QLabel *blue_label = new QLabel("Blue:");
+    layout->addWidget(blue_label, 3, 0, 1, 2);
+    QSpinBox *blue_spinbox = new QSpinBox();
+    blue_spinbox->setRange(0, 100);
+    blue_spinbox->setValue(_init_data.video_state.crosstalk_b * 100.0f);
+    layout->addWidget(blue_spinbox, 3, 2, 1, 4);
+    QPushButton *ok_button = new QPushButton(tr("&OK"));
+    ok_button->setDefault(true);
+    connect(ok_button, SIGNAL(clicked()), dialog, SLOT(accept()));
+    layout->addWidget(ok_button, 4, 0, 1, 3);
+    QPushButton *cancel_button = new QPushButton(tr("&Cancel"), dialog);
+    connect(cancel_button, SIGNAL(clicked()), dialog, SLOT(reject()));
+    layout->addWidget(cancel_button, 4, 3, 1, 3);
+    dialog->setLayout(layout);
+    if (dialog->exec() == QDialog::Rejected)
+    {
+        return;
+    }
+    _init_data.video_state.crosstalk_r = red_spinbox->value() / 100.0f;
+    _init_data.video_state.crosstalk_g = green_spinbox->value() / 100.0f;
+    _init_data.video_state.crosstalk_b = blue_spinbox->value() / 100.0f;
+    delete dialog;
+}
+
+void main_window::help_manual()
+{
+    QUrl manual_url;
+#if defined(Q_WS_WIN32)
+    manual_url = QUrl::fromLocalFile(QCoreApplication::applicationDirPath() + "/../doc/bino.html");
+#elif defined(Q_WS_MAC)
+    manual_url = QUrl::fromLocalFile(QCoreApplication::applicationDirPath() + "/../Resources/Bino Help/bino.html");
+#else
+    manual_url = QUrl::fromLocalFile(DOCDIR "/bino.html");
+#endif
+    if (!QDesktopServices::openUrl(manual_url))
+    {
+        QMessageBox::critical(this, "Error", "Cannot open manual");
+    }
+}
+
+void main_window::help_website()
+{
+    if (!QDesktopServices::openUrl(QUrl(PACKAGE_URL)))
+    {
+        QMessageBox::critical(this, "Error", "Cannot open website");
+    }
+}
+
 void main_window::help_keyboard()
 {
     QMessageBox::information(this, tr("Keyboard Shortcuts"), tr(
@@ -875,6 +1147,8 @@ void main_window::help_keyboard()
                 "<tr><td>3, 4</td><td>Adjust brightness</td></tr>"
                 "<tr><td>5, 6</td><td>Adjust hue</td></tr>"
                 "<tr><td>7, 8</td><td>Adjust saturation</td></tr>"
+                "<tr><td><, ></td><td>Adjust parallax</td></tr>"
+                "<tr><td>(, )</td><td>Adjust ghostbusting</td></tr>"
                 "<tr><td>left, right</td><td>Seek 10 seconds backward / forward</td></tr>"
                 "<tr><td>up, down</td><td>Seek 1 minute backward / forward</td></tr>"
                 "<tr><td>page up, page down</td><td>Seek 10 minutes backward / forward</td></tr>"
@@ -889,7 +1163,7 @@ void main_window::help_about()
 {
     QString blurb = tr(
             "<p>%1 version %2</p>"
-            "<p>Copyright (C) 2010 the Bino developers.<br>"
+            "<p>Copyright (C) 2011 the Bino developers.<br>"
             "This is free software. You may redistribute copies of it<br>"
             "under the terms of the <a href=\"http://www.gnu.org/licenses/gpl.html\">"
             "GNU General Public License</a>.<br>"
@@ -907,6 +1181,19 @@ void main_window::help_about()
     QMessageBox::about(this, tr("About " PACKAGE_NAME), blurb);
 }
 
+bool main_window::eventFilter(QObject *obj, QEvent *event)
+{
+    if (event->type() == QEvent::FileOpen)
+    {
+        open(QStringList(static_cast<QFileOpenEvent *>(event)->file()), true);
+        return true;
+    }
+    else
+    {
+        // pass the event on to the parent class
+        return QMainWindow::eventFilter(obj, event);
+    }
+}
 
 player_qt::player_qt() : player(player::slave)
 {
