@@ -1,7 +1,9 @@
 /*
  * This file is part of bino, a 3D video player.
  *
- * Copyright (C) 2010  Martin Lambers <marlam@marlam.de>
+ * Copyright (C) 2010-2011
+ * Martin Lambers <marlam@marlam.de>
+ * Frédéric Devernay <Frederic.Devernay@inrialpes.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +26,9 @@
 #include <QWidget>
 #include <QComboBox>
 #include <QPushButton>
+#include <QLabel>
+#include <QDoubleSpinBox>
+#include <QSpinBox>
 #include <QTimer>
 #include <QSettings>
 
@@ -60,10 +65,15 @@ class in_out_widget : public QWidget, public controller
 private:
     QSettings *_settings;
     QComboBox *_input_combobox;
+    QSpinBox *_audio_spinbox;
     QComboBox *_output_combobox;
     QPushButton *_swap_eyes_button;
     QPushButton *_fullscreen_button;
     QPushButton *_center_button;
+    QLabel *_parallax_label;
+    QDoubleSpinBox *_parallax_spinbox;
+    QLabel *_ghostbust_label;
+    QSpinBox *_ghostbust_spinbox;
     bool _lock;
 
     void set_input(enum input::mode m);
@@ -74,6 +84,8 @@ private slots:
     void swap_eyes_changed();
     void fullscreen_pressed();
     void center_pressed();
+    void parallax_changed();
+    void ghostbust_changed();
 
 public:
     in_out_widget(QSettings *settings, QWidget *parent);
@@ -82,6 +94,7 @@ public:
     void update(const player_init_data &init_data, bool have_valid_input, bool playing);
 
     enum input::mode input_mode();
+    int audio_stream();
     enum video_output::mode video_mode();
 
     virtual void receive_notification(const notification &note);
@@ -92,6 +105,7 @@ class controls_widget : public QWidget, public controller
     Q_OBJECT
 
 private:
+    bool _lock;
     QSettings *_settings;
     QPushButton *_play_button;
     QPushButton *_pause_button;
@@ -102,6 +116,7 @@ private:
     QPushButton *_f_button;
     QPushButton *_ff_button;
     QPushButton *_fff_button;
+    QSlider *_seek_slider;
     bool _playing;
 
 private slots:
@@ -114,6 +129,7 @@ private slots:
     void f_pressed();
     void ff_pressed();
     void fff_pressed();
+    void seek_slider_changed();
 
 public:
     controls_widget(QSettings *settings, QWidget *parent);
@@ -137,6 +153,7 @@ private:
     player_init_data _init_data;
     bool _stop_request;
 
+    QString current_file_hash();
     bool open_player();
     void open(QStringList filenames, bool automatic);
 
@@ -145,12 +162,16 @@ private slots:
     void playloop_step();
     void file_open();
     void file_open_url();
+    void preferences_crosstalk();
+    void help_manual();
+    void help_website();
     void help_keyboard();
     void help_about();
 
 protected:
     void moveEvent(QMoveEvent *event);
     void closeEvent(QCloseEvent *event);
+    bool eventFilter(QObject *obj, QEvent *event);
 
 public:
     main_window(QSettings *settings, const player_init_data &init_data);
