@@ -1,7 +1,8 @@
 /*
  * This file is part of bino, a 3D video player.
  *
- * Copyright (C) 2010  Martin Lambers <marlam@marlam.de>
+ * Copyright (C) 2010-2011
+ * Martin Lambers <marlam@marlam.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -57,6 +58,7 @@ protected:
     virtual void resizeGL(int width, int height);
     virtual void moveEvent(QMoveEvent *event);
     virtual void keyPressEvent(QKeyEvent *event);
+    virtual void mouseReleaseEvent(QMouseEvent *event);
 };
 
 /* Public interface. You can use this as a video container widget, to
@@ -89,6 +91,7 @@ private:
     void center();
     void enter_fullscreen();
     void exit_fullscreen();
+    void mouse_set_pos(float dest);
 
 protected:
     virtual int screen_pos_x();
@@ -111,7 +114,7 @@ public:
     virtual bool supports_stereo();
 
     virtual void open(
-            enum decoder::video_frame_format preferred_format,
+            int src_format, bool src_mono,
             int src_width, int src_height, float src_aspect_ratio,
             int mode, const video_output_state &state, unsigned int flags,
             int win_width, int win_height);
